#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class Api::V1::AlarmsController < Api::V1::BaseController
  # ----------------------------------------------
  # Before and after actions
  # ----------------------------------------------
  before_action :authenticate_user_from_token!, :user_setup_api, raise: false

  # ----------------------------------------------
  # Actions
  # ----------------------------------------------

  #
  # GET /api/v1/alarms/index
  #
  def index
    alarms = if current_user.super_admin?
              Alarm.all
            else
              domains_with_permission = current_user.domains_with_permission.pluck(:id)
              Alarm.all.select do |alarm|
                domains_with_permission.include?(alarm.sensor_id) &&
                current_user.modules.include?(alarm.product_type)
              end
            end

    alarms = alarms.select do |a|
      Redborder::Module.keys_with_interface.include?(a.product_type.to_sym)
    end

    if params[:fired].present? && params[:fired] == "true"
      alarms = alarms.select(&:fired).sort_by { |a| -a.last_fired.to_i }
    end

    render json: alarms.map { |a|
      {
        id: a.id,
        name: a.name,
        product_type: a.product_type,
        sensor_id: a.sensor_id,
        fired: a.fired,
        last_fired: a.last_fired,
        created_at: a.created_at,
        updated_at: a.updated_at
      }
    }
  end
end
