#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class Api::V1::MonitorController < Api::V1::BaseController

    include Redborder::Rbdruid
    include MonitorHelper

    before_action :set_custom_tab, only: [:update]

    skip_before_action :authenticate_user!, only: [:get_bulkstats_columns], raise: false
    skip_before_action :authenticate_user_from_token!, only: [:get_bulkstats_columns]
    #before_action :validate_cert, only: :import

    #
    def index
    end

    def read_file(file_name)
      file = File.open(file_name, "r")
      data = file.read
      file.close
      return data
    end

    #
    def get_bulkstats_columns
        #compare the md5 with the one we have, if its different send
        file_path = "/share/bulkstats.tar.gz"

        # If file doesnt exists
        # we cannot send it
        return error_503("#{I18n.t 'file_serve'}") if !File.exist?(file_path)

        #Get the file
        send_file file_path
    end

    def historicals
    result = {}

    begin
      cluster = Redborder::Cluster.instance
      historicals = cluster.historicals.sort_by(&:tier)
      tiers = cluster.tiers.sort_by(&:name)
      result[:historicals] = historicals.map(&:as_json)
      result[:tiers] = tiers.map(&:as_json)
    rescue => e
      return render json: { error: 'zookeeper_error', message: e.message }, status: :service_unavailable
    end

    begin
      managers = Manager.all.sort_by(&:name)
      first_manager = managers.first

      unless first_manager && first_manager.node
        return render json: { error: 'missing_manager_node_data' }, status: :unprocessable_entity
      end

      hosts = first_manager.node.dig('redborder', 'managers_per_services', 'druid-historical')
      return render json: { error: 'no_hosts_found' }, status: :not_found if hosts.nil?

      nodes_in_tiers = check_all_hosts_used_in_historical_nodes(tiers, hosts)
      historicals_tiers_with_nodes = nodes_in_tiers.reject { |e| e.nil? || e.empty? || e[:tier].nil? }
                                                  .map { |e| e[:tier] }.uniq.count

      colors = {}
      hosts.each_with_index { |node, i| colors[node] = color_palette(i) }

      result[:hosts] = hosts
      result[:nodes_in_tiers] = nodes_in_tiers
      result[:historicals_tiers_with_nodes] = historicals_tiers_with_nodes
      result[:colors] = colors
    rescue => e
      return render json: { error: 'host_processing_error', message: e.message }, status: :internal_server_error
    end

    begin
      errors = druid_historical_get_all_errors(hosts, cluster)
      result[:errors] = errors
      result[:historical_errors] = errors.select { |k, _| k.to_s.include?("hist") }
      result[:tier_errors] = errors.select { |k, _| k.to_s.include?("tier") }
    rescue => e
      return render json: { error: 'druid_error', message: e.message }, status: :internal_server_error
    end

    render json: result, status: :ok
  end
end
